'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = SyntaxHighlighter;

var _lowlight = require('lowlight');

var _lowlight2 = _interopRequireDefault(_lowlight);

var _defaultStyle = require('highlight.js-js-styles/dist/styles/default-style');

var _defaultStyle2 = _interopRequireDefault(_defaultStyle);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function assignStyleToElement(style, element) {
  Object.keys(style).forEach(function (styleKey) {
    return element.style[styleKey] = style[styleKey];
  });
}

function assignPropsToElement(props, element) {
  Object.keys(props).forEach(function (propKey) {
    if (propKey === 'style') {
      assignStyleToElement(props.style, element);
    } else {
      element.setAttribute(propKey, props[propKey]);
    }
  });
}

function createStyleObject(classNames, style) {
  return classNames.reduce(function (styleObject, className) {
    return _extends({}, styleObject, style[className]);
  }, {});
}

function createChildren(style, element) {
  return function (children, element) {
    children.forEach(function (child) {
      var childElement = createElement({ node: child, style: style });
      element.appendChild(childElement);
    });
  };
}

function createElement(_ref) {
  var node = _ref.node;
  var style = _ref.style;
  var properties = node.properties;
  var type = node.type;
  var tagName = node.tagName;
  var value = node.value;

  if (type === 'text') {
    var element = document.createElement('span');
    element.innerText = value;
    return element;
  } else if (tagName) {
    var childrenCreator = createChildren(style);
    var elementStyle = createStyleObject(properties.className, style);
    var _element = document.createElement(tagName);
    assignStyleToElement(elementStyle, _element);
    childrenCreator(node.children, _element);
    return _element;
  }
}

function SyntaxHighlighter(props) {
  var language = props.language;
  var codeString = props.codeString;
  var _props$style = props.style;
  var style = _props$style === undefined ? _defaultStyle2.default : _props$style;
  var _props$customStyle = props.customStyle;
  var customStyle = _props$customStyle === undefined ? {} : _props$customStyle;
  var _props$codeTagProps = props.codeTagProps;
  var codeTagProps = _props$codeTagProps === undefined ? {} : _props$codeTagProps;
  var querySelector = props.querySelector;

  var rest = _objectWithoutProperties(props, ['language', 'codeString', 'style', 'customStyle', 'codeTagProps', 'querySelector']);

  var codeTree = language ? _lowlight2.default.highlight(language, codeString) : _lowlight2.default.highlightAuto(codeString);
  var defaultPreStyle = style.hljs || { backgroundColor: '#fff' };
  var preProps = Object.assign({}, rest, { style: Object.assign({}, defaultPreStyle, customStyle) });
  var pre = document.createElement('pre');
  assignPropsToElement(preProps, pre);
  var code = document.createElement('code');
  assignPropsToElement(codeTagProps, code);

  codeTree.value.forEach(function (node) {
    var childElement = createElement({ node: node, style: style });
    code.appendChild(childElement);
  });
  pre.appendChild(code);
  var elementToAttachTo = querySelector && document.querySelector(querySelector);
  if (!elementToAttachTo) {
    document.body.appendChild(pre);
  } else {
    elementToAttachTo.appendChild(pre);
  }
}